using System;
using System.Collections;
using System.Reflection;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Drawing;
using System.Drawing.Design;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

namespace Wrox.Samples.GUI
{
  /// <summary>
  /// Specifies the positioning of labels in an AddressControl relative to the Address ExtTextBoxes
  /// </summary>
  public enum AddressLabelAlignment 
  {
    /// <summary>
    /// Labels in an AddressControl are positioned to the left of the ExtTextBoxes
    /// </summary>
    Left = 0, 
    
    /// <summary>
    /// Labels in an AddressControl are positioned to the top of the ExtTextBoxes
    /// </summary>
    Top = 1
  };

  /// <summary>Composite Address UserControl Example</summary>
  [Description("Composite Control Example"),
  Designer(typeof(AddressControl.AddressControlDesigner))]
  public class AddressControl : System.Windows.Forms.UserControl
  {
    private Wrox.Samples.GUI.ExtTextBox txtLine1;
    private Wrox.Samples.GUI.ExtTextBox txtLine2;
    private Wrox.Samples.GUI.ExtTextBox txtCity;
    private Wrox.Samples.GUI.ExtTextBox txtState;
    private Wrox.Samples.GUI.ExtTextBox txtZipCode;

    private System.Windows.Forms.Label lblAddress;
    private System.Windows.Forms.Label lblCity;
    private System.Windows.Forms.Label lblState;
    private System.Windows.Forms.Label lblZip;

    private AddressLabelAlignment mLabelAlignment;
    private System.EventHandler onLabelAlignmentChanged;

    private IAddressCollection _dataSource;

    /// <summary>
    /// Required designer variable.
    /// </summary>
    private System.ComponentModel.Container components = null;

    /// <summary>
    /// Initializes a new instance of the Wrox.Samples.GUI.AddressControl class
    /// </summary>
    public AddressControl()
    {
      InitializeComponent();

      txtLine1.Tag = "Address Line 1";
      txtLine2.Tag = "Address Line 2";
      txtCity.Tag = "City";
      txtState.Tag = "State";
      txtZipCode.Tag = "Zip Code";

      txtLine1.IsRequired = true;
      txtLine2.IsRequired = false;
      txtCity.IsRequired = true;
      txtState.IsRequired = true;
      txtZipCode.IsRequired = true;

      mLabelAlignment = AddressLabelAlignment.Left;
    }
    
    /// <summary>
    /// Gets or sets the IAddress based datasource for the control.
    /// </summary>
    [Category("DataBindings"),
    Description("Source of data for DataBinding. Must Implement IAddress")]
    public IAddressCollection DataSource {
      get {return _dataSource;}
      set {
        if(value!=null) {
          _dataSource=value;
          txtLine1.DataBindings.Add("Text",_dataSource,"AddressLine1");
          txtLine2.DataBindings.Add("Text",_dataSource,"AddressLine2");
          txtCity.DataBindings.Add("Text",_dataSource,"City");
          txtState.DataBindings.Add("Text",_dataSource,"State");
          txtZipCode.DataBindings.Add("Text",_dataSource,"Zip");
        }
      }
    }

    /// <summary>
    /// Gets or sets the text in the Address Line 1 ExtTextBox Control.
    /// </summary>
    [Category("Address"),
    Description("Gets or sets the text in the Address Line 1 ExtTextBox Control.")]
    public string AddressLine1Text
    {
      get {return this.txtLine1.Text;}
      set {this.txtLine1.Text = value;}
    }

    /// <summary>
    /// Occurs when the AddressControl.AddressLine1Text property value changes.
    /// </summary>
    [Category("Property Changed"),
    Description("Occurs when the AddressLine1Text property value changes.")]
    public event EventHandler AddressLine1TextChanged
    {
      add {this.txtLine1.TextChanged += value;}
      remove {this.txtLine1.TextChanged -= value;}
    }

    /// <summary>
    /// Gets or sets the text in the Address Line 2 ExtTextBox Control.
    /// </summary>
    [Category("Address"),
    Description("Gets or sets the text in the Address Line 2 ExtTextBox Control.")]
    public string AddressLine2Text
    {
      get {return this.txtLine2.Text;}
      set {this.txtLine2.Text = value;}
    }

    /// <summary>
    /// Occurs when the AddressControl.AddressLine2Text property value changes.
    /// </summary>
    [Category("Property Changed"),
    Description("Occurs when the AddressLine2Text property value changes.")]
    public event EventHandler AddressLine2TextChanged
    {
      add {this.txtLine2.TextChanged += value;}
      remove {this.txtLine2.TextChanged -= value;}
    }

    /// <summary>
    /// Gets or sets the text in the City ExtTextBox Control.
    /// </summary>
    [Category("Address"),
    Description("Gets or sets the text in the City ExtTextBox Control.")]
    public string CityText
    {
      get {return this.txtCity.Text;}
      set {this.txtCity.Text = value;}
    }

    /// <summary>
    /// Occurs when the AddressControl.CityText property value changes.
    /// </summary>
    [Category("Property Changed"),
    Description("Occurs when the CityText property value changes.")]
    public event EventHandler CityTextChanged
    {
      add {this.txtCity.TextChanged += value;}
      remove {this.txtCity.TextChanged -= value;}
    }

    /// <summary>
    /// Gets or sets the text in the State ExtTextBox Control.
    /// </summary>
    [Category("Address"),
    Description("Gets or sets the text in the State ExtTextBox Control.")]
    public string StateText
    {
      get {return this.txtState.Text;}
      set {this.txtState.Text = value;}
    }

    /// <summary>
    /// Occurs when the AddressControl.StateText property value changes.
    /// </summary>
    [Category("Property Changed"),
    Description("Occurs when the StateText property value changes.")]
    public event EventHandler StateTextChanged
    {
      add {this.txtState.TextChanged += value;}
      remove {this.txtState.TextChanged -= value;}
    }
    
    /// <summary>
    /// Gets or sets the text in the Zip Code ExtTextBox Control.
    /// </summary>
    [Category("Address"),
    Description("Gets or sets the text in the Zip Code ExtTextBox Control.")]
    public string ZipCodeText
    {
      get {return this.txtZipCode.Text;}
      set {this.txtZipCode.Text = value;}
    }

    /// <summary>
    /// Occurs when the AddressControl.ZipCodeText property value changes.
    /// </summary>
    [Category("Property Changed"),
    Description("Occurs when the ZipCodeText property value changes.")]
    public event EventHandler ZipCodeTextChanged
    {
      add {this.txtZipCode.TextChanged += value;}
      remove {this.txtZipCode.TextChanged -= value;}
    }
    
    /// <summary>
    /// The background color used to display text in all Address ExtTextBoxes displayed by the control.
    /// </summary>
    [Category("Address"),
    Description("Gets or sets the background color used to display text in all Address ExtTextBoxes displayed by the control."),
    DefaultValue(typeof(Color), "Window")]
    public Color AddressBackColor
    {
      get {return txtLine1.BackColor;}
      set 
      {
        if (!this.txtLine1.BackColor.Equals(value))
        {
          SetAddressProperty("BackColor", value);
        }
      }
    }

    /// <summary>
    /// Occurs when the AddressControl.AddressBackColor property value changes.
    /// </summary>
    [Category("Property Changed"),
    Description("Occurs when the AddressBackColor property value changes.")]
    public event EventHandler AddressBackColorChanged
    {
      add {this.txtLine1.BackColorChanged += value;}
      remove {this.txtLine1.BackColorChanged -= value;}
    }

    /// <summary>
    /// The foreground color used to display text in all Address ExtTextBoxes displayed by the control.
    /// </summary>
    [Category("Address"),
    Description("Gets or sets the foreground color used to display text in all Address ExtTextBoxes displayed by the control."),
    DefaultValue(typeof(Color), "WindowText")]
    public Color AddressForeColor
    {
      get {return txtLine1.ForeColor;}
      set 
      {
        if (!this.txtLine1.ForeColor.Equals(value))
        {
          SetAddressProperty("ForeColor", value);
        }
      }
    }

    /// <summary>
    /// Occurs when the AddressControl.AddressForeColor property value changes.
    /// </summary>
    [Category("Property Changed"),
    Description("Occurs when the AddressForeColor property value changes.")]
    public event EventHandler AddressForeColorChanged
    {
      add {this.txtLine1.ForeColorChanged += value;}
      remove {this.txtLine1.ForeColorChanged -= value;}
    }

    /// <summary>
    /// Gets the collection of controls contained within the control.
    /// </summary>    
    /// <remarks>
    /// Manipulating the constituent controls of the AddressControl directly via
    /// the Controls property can interfere with the encapsulated functionality 
    /// of the AddressControl. Please manipulate the constituent controls through the 
    /// publicly defined accessors specifically built for this purpose.
    /// </remarks>
    [EditorBrowsable(EditorBrowsableState.Never)]
    new public Control.ControlCollection Controls
    {
      get {return base.Controls;}
    }

    /// <summary>
    /// Gets or sets the foreground color of the control.
    /// </summary>    
    /// <remarks>
    /// This property is not used by the AddressControl. 
    /// </remarks>
    [EditorBrowsable(EditorBrowsableState.Never)]
    new public Color ForeColor
    {
      get {return base.ForeColor;}
      set {base.ForeColor = value;}
    }

    /// <summary>
    /// Gets or sets the Font used to display text in the control.
    /// </summary>    
    /// <remarks>
    /// This property is not used by the AddressControl. 
    /// </remarks>
    [EditorBrowsable(EditorBrowsableState.Never)]
    new public System.Drawing.Font Font
    {
      get {return base.Font;}
      set {base.Font = value;}
    }

    /// <summary>
    /// Gets a value indicating whether the Address information passes all specified validation rules.
    /// Returns a simple true or false value without displaying any exception messages.
    /// </summary>
    [Category("Address"),
    Description("Gets a value indicating whether the Address information passes all specified validation rules. Returns a simple true or false value without displaying any exception messages."),
    Browsable(false)]
    public bool IsAddressValid
    {
      get {return this.ValidateAll(false);}
    }
    
    /// <summary>
    /// Causes the control to validate the Address information and display 
    /// any appropriate exception message.
    /// </summary>
    public void ValidateAddress() 
    {
      this.ValidateAll(true);  
    }

    // Validate all concerned ExtTextBoxes
    private bool ValidateAll(bool DisplayErrors)
    { 
      try
      {
        ValidateExtTextBox(this.txtLine1);          
        ValidateExtTextBox(this.txtLine2);    
        ValidateExtTextBox(this.txtCity);    
        ValidateExtTextBox(this.txtState);    
        ValidateExtTextBox(this.txtZipCode);    
      }
      catch (Exception ex)
      {
        if (DisplayErrors)
        {
          MessageBox.Show(this, ex.Message, "Invalid Field");        
        }
        return false;      
      }
      return true;
    }

    // Validate an ExtTextBox if IsRequired == true
    private void ValidateExtTextBox(ExtTextBox BoxToTest)
    {
      if (BoxToTest.IsRequired && BoxToTest.Text.Length == 0)
      {
        BoxToTest.Focus();
        throw new Exception(String.Format("The {0} field is required.", BoxToTest.Tag));
      }
    }

    /// <summary>
    /// Controls whether the the Address can be changed by the user or not.
    /// </summary>
    [Category("Behavior"),
    Description("Gets or sets a value indicating whether the Address can be changed by the user or not."),
    DefaultValue(false)]
    public bool ReadOnly
    {
      get {return txtLine1.ReadOnly;}
      set {SetAddressProperty("ReadOnly", value);}
    }

    /// <summary>
    /// Occurs when the AddressControl.ReadOnly property value changes.
    /// </summary>
    [Category("Property Changed"),
    Description("Occurs when the Address.ReadOnly property value changes.")]
    public event EventHandler ReadOnlyChanged
    {
      add {this.txtLine1.ReadOnlyChanged += value;}
      remove {this.txtLine1.ReadOnlyChanged -= value;}
    }

    /// <summary>
    /// Clears the Address (that is to say the text contents of each 
    /// ExtTextBox control contained within this AddressControl).
    /// </summary>    
    public void Clear()
    {
      SetAddressProperty("Text", null);      
    }

    /// <summary>
    /// Gets or sets the alignment of the labels in relation to the Address ExtTextBoxes.
    /// </summary>
    [Category("Labels"),
    Description("Gets or sets the alignment of the labels in relation to the Address ExtTextBoxes."),
    DefaultValue(AddressLabelAlignment.Left)]
    public AddressLabelAlignment LabelAlignment
    {
      get {return mLabelAlignment;}
      set 
      {
        if (mLabelAlignment != value)
        {
          mLabelAlignment = value;
          this.SuspendLayout();
          if (mLabelAlignment == AddressLabelAlignment.Top)
          {
            this.lblAddress.Location = new System.Drawing.Point(0, 4);
            this.lblAddress.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
          
            this.lblCity.Location = new System.Drawing.Point(0, 72);
            this.lblCity.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
          
            this.lblState.Location = new System.Drawing.Point(0, 114);
            this.lblState.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
          
            this.lblZip.Location = new System.Drawing.Point(134, 114);
            this.lblZip.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;

            this.txtLine1.Location = new System.Drawing.Point(0, 22);
            this.txtLine2.Location = new System.Drawing.Point(0, 46);
            this.txtCity.Location = new System.Drawing.Point(0, 88);
            this.txtState.Location = new System.Drawing.Point(0, 130);
            this.txtZipCode.Location = new System.Drawing.Point(134, 130);

            this.Size = new System.Drawing.Size(240, 152);
          }
          else  // Default to labels left
          {
            this.lblAddress.Location = new System.Drawing.Point(0, 6);
            this.lblAddress.TextAlign = System.Drawing.ContentAlignment.MiddleRight;

            this.lblCity.Location = new System.Drawing.Point(22, 62);
            this.lblCity.TextAlign = System.Drawing.ContentAlignment.MiddleRight;

            this.lblState.Location = new System.Drawing.Point(12, 90);
            this.lblState.TextAlign = System.Drawing.ContentAlignment.MiddleRight;

            this.lblZip.Location = new System.Drawing.Point(160, 90);
            this.lblZip.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
          
            this.txtLine1.Location = new System.Drawing.Point(54, 4);
            this.txtLine2.Location = new System.Drawing.Point(54, 32);
            this.txtCity.Location = new System.Drawing.Point(54, 60);
            this.txtState.Location = new System.Drawing.Point(54, 88);
            this.txtZipCode.Location = new System.Drawing.Point(186, 88);
          
            this.Size = new System.Drawing.Size(296, 112);
          }
          this.ResumeLayout(false);        
          OnLabelAlignmentChanged(EventArgs.Empty);      
        }
      }
    }

    /// <summary>
    /// Occurs when the AddressControl.LabelAlignment property value changes.
    /// </summary>
    [Category("Property Changed"),
    Description("Occurs when the LabelAlignment property value changes.")]
    public event EventHandler LabelAlignmentChanged
    {
      add {onLabelAlignmentChanged += value;}
      remove {onLabelAlignmentChanged -= value;}
    }

    /// <summary>
    /// Raises the LabelAlignmentChanged event.
    /// </summary>
    /// <param name="e"> A System.EventArgs object that contains the event data.</param>
    protected virtual void OnLabelAlignmentChanged(EventArgs e) 
    {
      if (onLabelAlignmentChanged != null) 
      {
        onLabelAlignmentChanged(this, e);
      }
    }

    /// <summary>
    /// The background color used to display text in all labels displayed by the control.
    /// </summary>
    [Category("Labels"),
    Description("Gets or sets the background color of the text in all labels displayed by the control."),
    DefaultValue(typeof(Color), "Control")]
    public Color LabelBackColor
    {
      get {return lblAddress.BackColor;}
      set 
      {
        if (this.lblAddress.BackColor != value)
        {
          SetLabelsProperty("BackColor", value);
        }
      }
    }

    /// <summary>
    /// Occurs when the AddressControl.LabelBackColor property value changes.
    /// </summary>
    [Category("Property Changed"),
    Description("Occurs when the LabelBackColor property value changes.")]
    public event EventHandler LabelBackColorChanged
    {
      add {this.lblAddress.BackColorChanged += value;}
      remove {this.lblAddress.BackColorChanged -= value;}
    }

    /// <summary>
    /// The foreground color used to display text in all labels displayed by the control.
    /// </summary>
    [Category("Labels"),
    Description("Gets or sets the foreground color of the text in all labels displayed by the control."),
    DefaultValue(typeof(Color), "ControlText")]
    public Color LabelForeColor
    {
      get {return lblAddress.ForeColor;}
      set 
      {
        if (this.lblAddress.ForeColor != value)
        {
          SetLabelsProperty("ForeColor", value);
        }
      }
    }

    /// <summary>
    /// Occurs when the AddressControl.LabelForeColor property value changes.
    /// </summary>
    [Category("Property Changed"),
    Description("Occurs when the LabelForeColor property value changes.")]
    public event EventHandler LabelForeColorChanged
    {
      add {this.lblAddress.ForeColorChanged += value;}
      remove {this.lblAddress.ForeColorChanged -= value;}
    }

    /// <summary>
    /// The font used to display text in all labels displayed by the control.
    /// </summary>
    [Category("Labels"),
    Description("Gets or sets the font of the text in all labels displayed by the control.")]
    public Font LabelFont
    {
      get {return lblAddress.Font;}
      set 
      {
        if (this.lblAddress.Font != value)
        {
          SetLabelsProperty("Font", value);
        }
      }
    }
    
    // PropertyDescriptor ShouldSerializeValue method looks for a 
    // "ShouldSerialize[propertyname]" method if it doesn't find a 
    // DefaultValue() attribute for a public member
    private bool ShouldSerializeLabelFont()
    {
      return (LabelFont != Control.DefaultFont);
    }

    // PropertyDescriptor ResetValue method looks for a 
    // "Reset[propertyname]" method if it doesn't find a 
    // DefaultValue() attribute for a public member    
    private void ResetLabelFont()
    {
      LabelFont = Control.DefaultFont;
    }

    /// <summary>
    /// Occurs when the AddressControl.LabelFont property value changes.
    /// </summary>
    [Category("Property Changed"),
    Description("Occurs when the LabelFont property value changes.")]
    public event EventHandler LabelFontChanged
    {
      add {this.lblAddress.FontChanged += value;}
      remove {this.lblAddress.FontChanged -= value;}
    }

    private void SetLabelsProperty(string PropertyName, object NewValue)
    {
      PropertyInfo pi = (typeof(Label)).GetProperty(PropertyName);
      pi.SetValue(this.lblAddress, NewValue, null);
      pi.SetValue(this.lblCity, NewValue, null);
      pi.SetValue(this.lblState, NewValue, null);
      pi.SetValue(this.lblZip, NewValue, null);
    }

    private void SetAddressProperty(string PropertyName, object NewValue)
    {
      PropertyInfo pi = (typeof(ExtTextBox)).GetProperty(PropertyName);
      pi.SetValue(this.txtLine1, NewValue, null);          
      pi.SetValue(this.txtLine2, NewValue, null);          
      pi.SetValue(this.txtCity, NewValue, null);          
      pi.SetValue(this.txtState, NewValue, null);          
      pi.SetValue(this.txtZipCode, NewValue, null);          
    }

    /// <summary>
    /// Returns a System.String that represents the current Wrox.Samples.GUI.AddressControl object.
    /// </summary>
    /// <returns>
    /// A System.String that represents the current Wrox.Samples.GUI.AddressControl object.
    /// </returns>
    public override string ToString()
    {
      StringBuilder myText = new StringBuilder(txtLine1.Text + "\n");
      if (!(txtLine2.Text.Length == 0))
      {
        myText.Append(txtLine2.Text + "\n");
      }
      myText.Append(txtCity.Text + ", ");
      myText.Append(txtState.Text + "  ");
      myText.Append(txtZipCode.Text);
      return myText.ToString().TrimEnd("\n, ".ToCharArray());
    }

    /// <summary>
    /// Clean up any resources being used.
    /// </summary>
    protected override void Dispose( bool disposing )
    {
      if( disposing )
      {
        if( components != null ) components.Dispose();
      }
      base.Dispose( disposing );
    }

    #region Component Designer generated code
    /// <summary>
    /// Required method for Designer support - do not modify 
    /// the contents of this method with the code editor.
    /// </summary>
    private void InitializeComponent()
    {
      this.txtLine1 = new Wrox.Samples.GUI.ExtTextBox();
      this.txtLine2 = new Wrox.Samples.GUI.ExtTextBox();
      this.txtCity = new Wrox.Samples.GUI.ExtTextBox();
      this.txtState = new Wrox.Samples.GUI.ExtTextBox();
      this.txtZipCode = new Wrox.Samples.GUI.ExtTextBox();
      this.lblAddress = new System.Windows.Forms.Label();
      this.lblCity = new System.Windows.Forms.Label();
      this.lblState = new System.Windows.Forms.Label();
      this.lblZip = new System.Windows.Forms.Label();
      this.SuspendLayout();
      // 
      // txtLine1
      // 
      this.txtLine1.BackColor = System.Drawing.SystemColors.Window;
      this.txtLine1.Location = new System.Drawing.Point(54, 4);
      this.txtLine1.Name = "txtLine1";
      this.txtLine1.ReadOnly = false;
      this.txtLine1.Size = new System.Drawing.Size(232, 20);
      this.txtLine1.TabIndex = 0;
      this.txtLine1.Text = "";
      // 
      // txtLine2
      // 
      this.txtLine2.BackColor = System.Drawing.SystemColors.Window;
      this.txtLine2.Location = new System.Drawing.Point(54, 32);
      this.txtLine2.Name = "txtLine2";
      this.txtLine2.ReadOnly = false;
      this.txtLine2.Size = new System.Drawing.Size(232, 20);
      this.txtLine2.TabIndex = 1;
      this.txtLine2.Text = "";
      // 
      // txtCity
      // 
      this.txtCity.BackColor = System.Drawing.SystemColors.Window;
      this.txtCity.Location = new System.Drawing.Point(54, 60);
      this.txtCity.Name = "txtCity";
      this.txtCity.ReadOnly = false;
      this.txtCity.Size = new System.Drawing.Size(232, 20);
      this.txtCity.TabIndex = 2;
      this.txtCity.Text = "";
      // 
      // txtState
      // 
      this.txtState.BackColor = System.Drawing.SystemColors.Window;
      this.txtState.Location = new System.Drawing.Point(54, 88);
      this.txtState.Name = "txtState";
      this.txtState.ReadOnly = false;
      this.txtState.Size = new System.Drawing.Size(68, 20);
      this.txtState.TabIndex = 3;
      this.txtState.Text = "";
      // 
      // txtZipCode
      // 
      this.txtZipCode.BackColor = System.Drawing.SystemColors.Window;
      this.txtZipCode.Location = new System.Drawing.Point(186, 88);
      this.txtZipCode.Name = "txtZipCode";
      this.txtZipCode.ReadOnly = false;
      this.txtZipCode.TabIndex = 4;
      this.txtZipCode.Text = "";
      // 
      // lblAddress
      // 
      this.lblAddress.AutoSize = true;
      this.lblAddress.Location = new System.Drawing.Point(0, 6);
      this.lblAddress.Name = "lblAddress";
      this.lblAddress.Size = new System.Drawing.Size(46, 13);
      this.lblAddress.TabIndex = 5;
      this.lblAddress.Text = "Address";
      this.lblAddress.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
      // 
      // lblCity
      // 
      this.lblCity.AutoSize = true;
      this.lblCity.Location = new System.Drawing.Point(22, 62);
      this.lblCity.Name = "lblCity";
      this.lblCity.Size = new System.Drawing.Size(24, 13);
      this.lblCity.TabIndex = 6;
      this.lblCity.Text = "City";
      this.lblCity.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
      // 
      // lblState
      // 
      this.lblState.AutoSize = true;
      this.lblState.Location = new System.Drawing.Point(12, 90);
      this.lblState.Name = "lblState";
      this.lblState.Size = new System.Drawing.Size(34, 13);
      this.lblState.TabIndex = 7;
      this.lblState.Text = "State ";
      this.lblState.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
      // 
      // lblZip
      // 
      this.lblZip.AutoSize = true;
      this.lblZip.Location = new System.Drawing.Point(160, 90);
      this.lblZip.Name = "lblZip";
      this.lblZip.Size = new System.Drawing.Size(20, 13);
      this.lblZip.TabIndex = 8;
      this.lblZip.Text = "Zip";
      this.lblZip.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
      // 
      // AddressControl
      // 
      this.Controls.AddRange(new System.Windows.Forms.Control[] {
                                                                  this.lblZip,
                                                                  this.lblState,
                                                                  this.lblCity,
                                                                  this.lblAddress,
                                                                  this.txtZipCode,
                                                                  this.txtState,
                                                                  this.txtCity,
                                                                  this.txtLine2,
                                                                  this.txtLine1});
      this.Name = "AddressControl";
      this.Size = new System.Drawing.Size(296, 112);
      this.ResumeLayout(false);

    }
    #endregion
   
    internal class AddressControlDesigner :  System.Windows.Forms.Design.ControlDesigner
    {
      private DesignerVerbCollection mVerbs = null;

      public AddressControlDesigner()
      {
      }

      private void OnVerbClearAddress(object sender, EventArgs e) 
      {
        ((Wrox.Samples.GUI.AddressControl)Control).Clear();
      }

      public override DesignerVerbCollection Verbs 
      {
        get 
        {
          if (mVerbs == null) 
          {
            mVerbs = new DesignerVerbCollection();
            mVerbs.Add(new DesignerVerb("Clear Address", new EventHandler(OnVerbClearAddress)));
          }
          return mVerbs;
        }
      }

      protected override void PostFilterProperties(IDictionary Properties)
      {
        Properties.Remove("ForeColor");
        Properties.Remove("Font");
        base.PostFilterProperties(Properties);
      }
      
      protected override void PostFilterEvents(IDictionary Events)
      {
        Events.Remove("ForeColorChanged");
        Events.Remove("FontChanged");
        base.PostFilterEvents(Events);
      }

    } 
  }
}
