using System;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;

namespace Wrox.Samples.GUI
{
  /// <summary>
  /// Simple extension of a standard Windows Forms TextBox
  /// </summary>
	
  public class ExtTextBox : System.Windows.Forms.TextBox
  {
    private Color mHoldColor;
    private bool mIsRequired;
    private System.EventHandler onIsRequiredChanged;

    /// <summary>
    /// Initializes a new instance of the Wrox.Samples.GUI.ExtTextBox class
    /// </summary>
    public ExtTextBox()
    {
    }

    /// <summary>
    /// Gets or sets a value indicating whether text in the control can be changed or not.
    /// </summary>
    /// <remarks>
    /// Replaces the ReadOnly property of TextBox. When ReadOnly is true, the BackColor 
    /// is grayed out as a visual cue to the user.
    /// </remarks>
    // public override bool ReadOnly is not allowed
    [Category("Behavior"),
    Description("Gets or sets a value indicating whether text in the " +
    "control can be changed or not. Also adjusts the BackColor property " +
    "accordingly as a visual cue to the user."),
    DefaultValue(false)]
    new public bool ReadOnly
    {
      get {return base.ReadOnly;}
      set
      {
        if (base.ReadOnly != value)
        {
          if (value)
          {
            // Push BackColor to hold color
            // before actually changing
            mHoldColor = base.BackColor;
            base.BackColor = SystemColors.Control;
          }
          else
          {
            // Pop BackColor from hold color
            base.BackColor = mHoldColor;  
          }
          base.ReadOnly = value;
        }
      }
    }

    /// <summary>
    /// Gets or sets the background color of the control.
    /// </summary>
    /// <remarks>
    /// Overriden from TextBox. If the ExtTextBox's ReadOnly property is true, 
    /// the BackColor is not actually changed, but is saved until such time
    /// as the ReadOnly property is set to false.
    /// </remarks>
    [Category("Appearance"),
    Description("Gets or sets the background color of the control."),
    DefaultValue(typeof(Color), "Control")]
    public override Color BackColor
    {
      get {return base.BackColor;}
      
      set
      {
        if (base.ReadOnly)
        {
          // Push Color value to hold color since
          // ReadOnly BackColor should stay LightGray
          mHoldColor = value;
        }
        else
        {
          // We are free to actually change the color
          base.BackColor = value;
        }
      }
    }

    /// <summary>
    /// Gets or sets a value indicating whether the control is required.
    /// </summary>
    [Category("Behavior"),
    Description("Gets or sets a value indicating whether the control " +
    "is required."),
    DefaultValue(false)]
    public bool IsRequired
    {
      get {return mIsRequired;}
      
      set
      {
        if (!mIsRequired.Equals(value)) 
        {
          mIsRequired = value;        
          OnIsRequiredChanged(EventArgs.Empty);
        }
      }
    }

    /// <summary>
    /// Occurs when the ExtTextBox.IsRequired property value changes.
    /// </summary>
    [Category("Property Changed"),
    Browsable(true),
    Description("Occurs when the IsRequired property value changes.")]
    public event EventHandler IsRequiredChanged
    {
      add {onIsRequiredChanged += value;}
      remove {onIsRequiredChanged -= value;}
    }
    
    /// <summary>
    /// Raises the ExtendedTextBox.ExtTextBox.IsRequiredChanged event.
    /// </summary>
    /// <param name="e"> 
    /// A System.EventArgs object that contains the event data.
    /// </param>
    protected virtual void OnIsRequiredChanged(EventArgs e) 
    {
      if (onIsRequiredChanged != null) 
      {
        onIsRequiredChanged(this, e);
      }
    }
  }
}