using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

namespace Wrox.Samples.GUI
{
	/// <summary>
	/// Summary description for RoundButton.
	/// </summary>
  public class RoundButton : System.Windows.Forms.Control, IButtonControl
  {
    private ButtonState  mButtonState;
    private DialogResult mDialogResult;
    private bool mIsDefault;
    
    /// <summary>
    /// Initializes a new instance of the Wrox.Samples.GUI.RoundButton class.
    /// </summary>
    public RoundButton()
    {
      mButtonState = ButtonState.Normal;
    }

    /// <summary>
    /// Raises the RoundButton.Paint event.
    /// </summary>
    /// <param name="p">
    /// A System.Windows.Forms.PaintEventArgs that contains the event data.
    /// </param>
    protected override void OnPaint(PaintEventArgs p)
    {
      base.OnPaint(p);
      PaintStandard(p.Graphics);
    }

    private void PaintStandard(Graphics g)
    {
      g.SmoothingMode = SmoothingMode.AntiAlias;
      Rectangle myRect = new Rectangle(2, 2, this.Width - 4, this.Height - 4);
      if (this.Focused)
      {
        //Shrink the rectangle to allow room for the focus border
        myRect.Inflate(-2,-2);
      }

      PaintButtonBackGround();
      Draw3DBorder(myRect, g);      
      DrawText(myRect, g);
      if (this.Focused)
      {
        DrawFocus(g);
      }
    }

    private void PaintButtonBackGround()
    {
      GraphicsPath myPath = new GraphicsPath();

      //Region is full width and height of control
      myPath.AddEllipse(0, 0, this.Width, this.Height);
      Region myRegion = new Region(myPath);
      this.Region = myRegion;  
    }

    private void Draw3DBorder(Rectangle r, Graphics g)
    {
      if (mButtonState == ButtonState.Pushed)
      {
        Draw3DBorderPushed(r, g);
      }
      else
      {
        Draw3DBorderNormal(r, g);
      }
    }

    private void Draw3DBorderNormal(Rectangle r, Graphics g)
    {
      LinearGradientBrush myBrush = new LinearGradientBrush(r, Color.White, Color.Black, LinearGradientMode.ForwardDiagonal);
      Pen myPen = new Pen(myBrush, 3);
      g.DrawEllipse(myPen, r);
      myPen.Dispose();
    }

    private void Draw3DBorderPushed(Rectangle r, Graphics g)
    {
      Pen myPen = new Pen(SystemColors.ControlDarkDark, 3);
      g.DrawEllipse(myPen, r);
      myPen.Dispose();
    }

    private void DrawText(Rectangle r, Graphics g)
    {
      StringFormat myFormat = new StringFormat();
      myFormat.Alignment = StringAlignment.Center;
      myFormat.LineAlignment = StringAlignment.Center;

      if (this.Enabled) 
      {
        if (mButtonState == ButtonState.Pushed)
        {
          r.Offset(1, 1);
        }
        g.DrawString(this.Text, this.Font, new SolidBrush(this.ForeColor), r, myFormat);    
      }
      else
      {
        r.Offset(1, 1);
        g.DrawString(this.Text, this.Font, new SolidBrush(Color.White), r, myFormat);    
        r.Offset(-1, -1);
        g.DrawString(this.Text, this.Font, new SolidBrush(SystemColors.GrayText), r, myFormat);    
      }
      r.Offset(0, 0);
    }

    private void DrawFocus(Graphics g)
    {
      //Focus rectangle the full width and height of the control
      Rectangle myRect = new Rectangle(1, 1, this.Width - 2, this.Height - 2);
      Pen myPen = new Pen(Color.Black, 2);
      g.DrawEllipse(myPen, myRect);
      myPen.Dispose();
    }
    
    /// <summary>
    /// Raises the RoundButton.MouseDown event.
    /// </summary>
    /// <param name="m">
    /// A System.Windows.Forms.MouseEventArgs that contains the event data.
    /// </param>
    protected override void OnMouseDown(MouseEventArgs m)
    {
      if (m.Button == MouseButtons.Left) 
      {
        this.Focus();
        this.Capture = true;
        mButtonState = ButtonState.Pushed;
        this.Invalidate();
        this.Update();
      }
      else
      {
        base.OnMouseDown(m);
      }
    }

    /// <summary>
    /// Raises the RoundButton.MouseUp event.
    /// </summary>
    /// <param name="m">
    /// A System.Windows.Forms.MouseEventArgs that contains the event data.
    /// </param>
    protected override void OnMouseUp(MouseEventArgs m)
    {
      if ((mButtonState == ButtonState.Pushed) && (m.Button == MouseButtons.Left)) 
      {
        this.Capture = false;
        mButtonState = ButtonState.Normal;
        this.Invalidate();
        this.Update();
      }
      else
      {
        base.OnMouseUp(m);
      }
    }

    /// <summary>
    /// Raises the RoundButton.GotFocus event.
    /// </summary>
    /// <param name="e"> A System.EventArgs that contains the event data.</param>
    protected override void OnGotFocus(EventArgs e)
    {
      this.Invalidate();
      base.OnGotFocus(e);
    }
    
    /// <summary>
    /// Raises the RoundButton.LostFocus event.
    /// </summary>
    /// <param name="e"> A System.EventArgs that contains the event data.</param>    
    protected override void OnLostFocus(EventArgs e)
    {
      this.Invalidate();
      base.OnLostFocus(e);
    }

    /// <summary>
    /// Raises the RoundButton.SizeChanged event.
    /// </summary>
    /// <param name="e"> A System.EventArgs that contains the event data.</param>
    protected override void OnSizeChanged(EventArgs e)
    {
      base.OnSizeChanged(e);
      this.Invalidate();
      this.Update();
    }    

    /// <summary>
    /// Raises the RoundButton.TextChanged event.
    /// </summary>
    /// <param name="e"> A System.EventArgs that contains the event data.</param>
    protected override void OnTextChanged(EventArgs e)
    {
      base.OnTextChanged(e);
      this.Invalidate();
      this.Update();
    }    

    /// <summary>
    /// Raises the RoundButton.Click event.
    /// </summary>
    /// <param name="e"> A System.EventArgs that contains the event data.</param>
    protected override void OnClick(EventArgs e)
    {
      base.OnClick(e); 
     ((Form)this.TopLevelControl).DialogResult = mDialogResult;
    }
    
    [Category("Behavior"),
    Description("Gets or sets the value returned to the parent form " 
    + "when the button is clicked."),
    DefaultValue(DialogResult.None)]
    public virtual DialogResult DialogResult
    {
      get {return mDialogResult;}
      set {mDialogResult = value;}
    }
    
    /// <summary>
    /// Notifies a control that it is the default button so that its appearance 
    /// and behavior is adjusted accordingly.
    /// </summary>
    /// <param name="value">
    /// true if the control should behave as a default button; otherwise false.
    /// </param>
    public void NotifyDefault(bool value)
    {
      mIsDefault = value;
      this.Invalidate();
      this.Update();
    }

    /// <summary>
    /// Generates a System.Windows.Forms.Control.Click event for the control.
    /// </summary>
    public void PerformClick()
    {
      this.OnClick(EventArgs.Empty);
    }
  }
}
